<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Client;

use ConnectPayPayment\Components\Client\Request\RequestInterface;
use ConnectPayPayment\Components\Client\Response\GenericResponse;
use ConnectPayPayment\Components\Service\ConfigService\ConfigServiceInterface;
use GuzzleHttp\Client as GuzzleClient;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Log\LoggerInterface;
use Shopware\Core\Framework\ShopwareException;
use Shopware\Core\Framework\Uuid\Uuid;
use Symfony\Component\HttpFoundation\Request;
use ConnectPayPayment\Components\Struct\Configuration;

class Client implements ClientInterface
{
    private GuzzleClient $restClient;
    private ConfigServiceInterface $configService;
    private LoggerInterface $logger;

    /**
     * @param ConfigServiceInterface $configService
     * @param LoggerInterface $logger
     */
    public function __construct(
        ConfigServiceInterface $configService,
        LoggerInterface $logger
    ) {
        $this->configService = $configService;
        $this->logger = $logger;

        $this->restClient = new GuzzleClient();
    }

    /**
     * {@inheritDoc}
     */
    public function request(RequestInterface $request): GenericResponse
    {
        $configuration = $this->configService->get();
        $debug = $configuration->get($this->configService::DEBUG_MODE);

        try {
            $requestData = [
                'headers' => $this->getHeaders($request, $configuration),
            ];

            if (Request::METHOD_POST === $request->getMethod()) {
                $requestData = array_merge($requestData, $request->getPostData());
            }

            if ($debug) {
                $requestData['debug'] = true;
                $this->logger->debug('Request: '.json_encode($requestData));
            }

            $gResponse = $this->restClient->request(
                $request->getMethod(),
                $request->getEndpoint(),
                $requestData
            );

            $responseBody = $gResponse->getBody()->getContents();

            $responseData = [
                'httpStatus' => $gResponse->getStatusCode(),
                'response' => json_decode($responseBody, true),
            ];

            if ($debug) {
                $this->logger->debug("Response: {$gResponse->getStatusCode()}: $responseBody");
            }

        } catch (GuzzleException $e) {
            $eResponse = $e->getResponse();
            $responseData = $eResponse
                ? [
                    'httpStatus' => $eResponse->getStatusCode(),
                    'response' => $eResponse->getBody()->getContents(),
                ]
                : [
                    'httpStatus' => $e->getCode(),
                    'response' => $e->getMessage(),
                ];

            $this->logger->error("{$responseData['httpStatus']}: {$responseData['response']}");
        } catch (ShopwareException $e) {
            $responseData = [
                'httpStatus' => $e->getCode(),
                'response' => $e->getMessage(),
            ];

            $this->logger->error("{$responseData['httpStatus']}: {$responseData['response']}");
        }

        $response = new GenericResponse();
        $response->assign($responseData);

        return $response;
    }

    /**
     * Build request headers
     *
     * @param RequestInterface $request
     * @param Configuration $configuration
     * @return array
     */
    private function getHeaders(RequestInterface $request, Configuration $configuration): array
    {
        return array_merge(
            [
                'X-Request-ID' => Uuid::randomHex(),
                'Accept' => 'application/json;version=2',
                'X-Signature' => $this->generateSignature($request->getMethod(), Request::METHOD_POST === $request->getMethod() ? $request->getPostData()['body'] : '', $request->getEndpoint(), $configuration),
            ],
            $request->getHeaders()
        );
    }
    
    /**
     * Generate request signature.
     * @param string $method
     * @param string $data
     * @param string $url
     * @param Configuration $configuration
     * @return string
     */
    private function generateSignature(string $method, string $data, string $url, Configuration $configuration): string
    {
        $signature = '';
        
        $cleanUrl = str_replace('https://', '', $url);
        $minifiedData = preg_replace('/\s+/', '', $data);
        $dataToSign = strtolower($method) . '|' . strtolower($cleanUrl) . '|' . $minifiedData;
        
        $pkeyid = openssl_get_privatekey($configuration->get($this->configService::PEM_KEY)); 

        openssl_sign($dataToSign, $signature, $pkeyid, OPENSSL_ALGO_SHA256);
        openssl_free_key($pkeyid);
        
        return base64_encode($signature);
    }
}
