<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Service\ConfigService;

use ConnectPayPayment\Components\Struct\Configuration;
use ConnectPayPayment\ConnectPayPayment;
use League\Flysystem\FilesystemInterface;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\MultiFilter;
use Shopware\Core\System\SystemConfig\SystemConfigService;

class ConfigService implements ConfigServiceInterface
{
    public FilesystemInterface $filesystem;
    private SystemConfigService $systemConfigService;
    private EntityRepositoryInterface $pluginRepository;

    /**
     * @param SystemConfigService $systemConfigService
     * @param EntityRepositoryInterface $pluginRepository
     * @param FilesystemInterface $filesystem
     */
    public function __construct(
        SystemConfigService $systemConfigService,
        EntityRepositoryInterface $pluginRepository,
        FilesystemInterface $filesystem
    ) {
        $this->systemConfigService = $systemConfigService;
        $this->pluginRepository = $pluginRepository;
        $this->filesystem = $filesystem;
    }

    /**
     * {@inheritDoc}
     */
    public function get(?string $salesChannelId = null): Configuration
    {
        $settings = $this->systemConfigService->get(self::BASE_PATH, $salesChannelId);

        return new Configuration(array_merge(...array_values($settings)));
    }

    /**
     * Set config domain value
     *
     * @param string $key
     * @param $value
     * @param string|null $salesChannelId
     * @return void
     */
    public function setConfig(string $key, $value, ?string $salesChannelId = null): void
    {
        $this->set($key, $value, self::CONFIG_PATH, $salesChannelId);
    }

    /**
     * {@inheritDoc}
     */
    public function set(string $key, $value, string $path, ?string $salesChannelId = null): void
    {
        $this->systemConfigService->set(self::BASE_PATH.".$path.$key", $value, $salesChannelId);
    }

    /**
     * {@inheritDoc}
     */
    public function unset(string $key, string $path, ?string $salesChannelId = null): void
    {
        $this->systemConfigService->set(self::BASE_PATH.".$path.$key", null, $salesChannelId);
    }

    /**
     * Unset config domain value
     *
     * @param string $key
     * @param string|null $salesChannelId
     * @return void
     */
    public function unsetConfig(string $key, ?string $salesChannelId = null): void
    {
        $this->unset($key, self::CONFIG_PATH, $salesChannelId);
    }

    /**
     * {@inheritDoc}
     */
    public function pluginActive(Context $context): bool
    {
        $criteria = new Criteria();
        $criteria->addFilter(
            new MultiFilter(MultiFilter::CONNECTION_AND, [
                new EqualsFilter('baseClass', ConnectPayPayment::class),
                new EqualsFilter('active', 1),
            ])
        );

        return (bool)$this->pluginRepository->searchIds($criteria, $context)->firstId();
    }
}
