<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Service\ProvidersService;

use Psr\Log\LoggerInterface;
use Shopware\Core\Framework\Context;
use Symfony\Component\HttpFoundation\Response;
use ConnectPayPayment\Components\Client\ClientInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use ConnectPayPayment\Components\Service\ConfigService\ConfigServiceInterface;
use ConnectPayPayment\Components\Client\Hydrator\Request\GetProviders\GetProvidersRequestHydratorInterface;

class ProvidersService implements ProvidersServiceInterface
{
    private EntityRepository $providersRepository;
    private EntityRepository $providersCountriesRepository;
    private GetProvidersRequestHydratorInterface $providersHydrator;
    private ClientInterface $client;
    private ConfigServiceInterface $configService;
    private LoggerInterface $logger;

    /**
     * @param EntityRepository $providersRepository
     * @param EntityRepository $providersCountriesRepository
     * @param GetProvidersRequestHydratorInterface $providersHydrator
     * @param ClientInterface $client
     * @param ConfigServiceInterface $configService
     * @param LoggerInterface $logger
     */
    public function __construct(
        EntityRepository $providersRepository,
        EntityRepository $providersCountriesRepository,
        GetProvidersRequestHydratorInterface $providersHydrator,
        ClientInterface $client,
        ConfigServiceInterface $configService,
        LoggerInterface $logger
    ) {
        $this->providersRepository = $providersRepository;
        $this->providersCountriesRepository = $providersCountriesRepository;
        $this->providersHydrator = $providersHydrator;
        $this->client = $client;
        $this->configService = $configService;
        $this->logger = $logger;
    }
    
    public function retrieveProviders(): bool
    {
        $firstRun = true;
        $providersData = [];
        $providersCntr = 0;
        $context = Context::createDefaultContext();
        $configuration = $this->configService->get();

        $this->clearCountries($context);
        
        foreach (self::PROVIDERS_COUNTRY_CODES as $countryCode => $countryName) {
            
            $pageNum = 1;
            
            $this->providersCountriesRepository->create([[
                'code' => $countryCode,
                'name' => $countryName,
            ]], $context);   
            
            if (in_array($countryCode, self::EXCLUDE_PROVIDERS_BY_COUNTRY)) continue;
            
            do { //loop to go throug pages if some exist
                
                $request = $this->providersHydrator->hydrate($countryCode, $configuration, $pageNum);
                $response = $this->client->request($request);

                if ($response && $response->getHttpStatus() === Response::HTTP_OK) {
                    if ($providers = $response->getResponse()['providers'] ?? []) {
                        
                        if ($firstRun) {
                            $firstRun = false;
                            $this->clearProviders($context);
                        }

                        $this->collectAndSave($providers, $providersCntr, $context);
                    }
                } else {
                    return false;
                }
                
                $pageNum++;
                
            } while ($response->getResponse()['_links']['next']['href']);
            
        }

        return true;
    }
    
    /**
     * Clear current providers data.
     * @param Context $context
     * @return void
     */
    private function clearProviders(Context $context): void
    {
        if ($ids = $this->providersRepository->search(new Criteria(), $context)->getIds()) {
            $this->providersRepository->delete($this->createDeleteIdsArray(array_values($ids)), $context);
        }
    }
    
    /**
     * Clear current providers countries data.
     * @param Context $context
     * @return void
     */
    private function clearCountries(Context $context): void
    {
        if ($ids = $this->providersCountriesRepository->search(new Criteria(), $context)->getIds()) {
            $this->providersCountriesRepository->delete($this->createDeleteIdsArray(array_values($ids)), $context);
        }
    }
    
    /**
     * Collect rpvoders array and save it to the repo.
     * @param array $providers
     * @param int $counter
     * @param Context $context
     * @return void
     */
    private function collectAndSave(array $providers, int &$counter, Context $context): void
    {
        foreach ($providers as $provider) {
            
            if ($provider['status']['availability'] !== self::PROVIDER_ACTIVE_STATUS) continue;
            
            $counter++;

            $providersData[] = [
                'providerId' => $provider['id'],
                'name' => $provider['name'],
                'countryCode' => $provider['countryCode'],
                'bic' => (!empty($provider['bic']) && is_array($provider['bic'])) ? json_encode($provider['bic']) : '',
                'logo' => $provider['presentation']['logo'] ?: '',
            ];

            if ($counter >= self::BULK_SIZE) {
                $this->providersRepository->create($providersData, $context);   
                $counter = 0;
                $providersData = [];
            }

        }
        
        $this->providersRepository->create($providersData, $context);   
    }
    
    /**
     * Create array of format ['id' => xx] to be used in repo delete query.
     * @param array $ids
     * @return array
     */
    private function createDeleteIdsArray(array $ids): array
    {
        return array_map(static function ($id) {
            return ['id' => $id];
        }, $ids);
    }
}
